<?php
/*
Plugin Name: Bricks Template Auto-Inserter
Description: Automatically insert Bricks templates between post paragraphs
Version: 1.4
Author: Oview Media
*/

/**
 * 11/10/2024 - 1.4 - Added New "Before/After Last Paragraph" Options + Ajax to hide the Insert After Paragraph # textfield.
 *            - 1.3 - Added dropdowns of available Bricks templates. 
 *                  - Added third Bricks template section.
 *              1.1 - Added second template section
 *                  - Made template renderers modular
 *                  - Improved settings organization
 *              1.0 - Initial release
 */

if (!defined('ABSPATH')) {
    exit;
}

// Define position types
define('BTAI_POSITION_AFTER_PARAGRAPH', 'after_paragraph');
define('BTAI_POSITION_BEFORE_LAST', 'before_last');
define('BTAI_POSITION_AFTER_LAST', 'after_last');

// Add admin menu item
add_action('admin_menu', 'btai_add_admin_menu', 11); // Priority 11 to ensure it runs after Bricks
add_action('admin_init', 'btai_settings_init');

function btai_add_admin_menu() {
    // Only add the menu if Bricks is active
    if (!class_exists('Bricks\Database')) {
        return;
    }

    add_submenu_page(
        'bricks',                     // Parent slug (Bricks menu)
        'Template Auto-Inserter',     // Page title
        'Auto-Insert',                // Menu title
        'manage_options',             // Capability
        'btai-settings',              // Menu slug
        'btai_options_page'           // Function
    );
}

// Helper function to get all Bricks templates
function btai_get_bricks_templates() {
    if (!class_exists('Bricks\Database')) {
        return array();
    }

    $templates = get_posts(array(
        'post_type'      => BRICKS_DB_TEMPLATE_SLUG,
        'posts_per_page' => -1,
        'orderby'        => 'title',
        'order'          => 'ASC',
    ));

    $template_options = array();
    foreach ($templates as $template) {
        $template_options[$template->ID] = $template->post_title;
    }

    return $template_options;
}

function btai_settings_init() {
    register_setting('btai_settings', 'btai_settings');

    // First Template Section
    add_settings_section(
        'btai_templates_section_1',
        'First Bricks Template',
        'btai_templates_section_1_callback',
        'btai_settings'
    );
    add_settings_field(
        'btai_template_id',
        'Select Template',
        'btai_template_dropdown_render',
        'btai_settings',
        'btai_templates_section_1',
        ['template_number' => 1]
    );
    add_settings_field(
        'btai_position_type',
        'Position Type',
        'btai_position_type_render',
        'btai_settings',
        'btai_templates_section_1',
        ['template_number' => 1]
    );
    add_settings_field(
        'btai_paragraph_number',
        'Insert After Paragraph #',
        'btai_paragraph_number_render',
        'btai_settings',
        'btai_templates_section_1',
        ['template_number' => 1]
    );
    add_settings_field(
        'btai_min_paragraphs',
        'Minimum Required Paragraphs',
        'btai_min_paragraphs_render',
        'btai_settings',
        'btai_templates_section_1',
        ['template_number' => 1]
    );

    // Second Template Section
    add_settings_section(
        'btai_templates_section_2',
        'Second Bricks Template',
        'btai_templates_section_2_callback',
        'btai_settings'
    );
    add_settings_field(
        'btai_template_id_2',
        'Select Template',
        'btai_template_dropdown_render',
        'btai_settings',
        'btai_templates_section_2',
        ['template_number' => 2]
    );
    add_settings_field(
        'btai_position_type_2',
        'Position Type',
        'btai_position_type_render',
        'btai_settings',
        'btai_templates_section_2',
        ['template_number' => 2]
    );
    add_settings_field(
        'btai_paragraph_number_2',
        'Insert After Paragraph #',
        'btai_paragraph_number_render',
        'btai_settings',
        'btai_templates_section_2',
        ['template_number' => 2]
    );
    add_settings_field(
        'btai_min_paragraphs_2',
        'Minimum Required Paragraphs',
        'btai_min_paragraphs_render',
        'btai_settings',
        'btai_templates_section_2',
        ['template_number' => 2]
    );

    // Third Template Section
    add_settings_section(
        'btai_templates_section_3',
        'Third Bricks Template',
        'btai_templates_section_3_callback',
        'btai_settings'
    );
    add_settings_field(
        'btai_template_id_3',
        'Select Template',
        'btai_template_dropdown_render',
        'btai_settings',
        'btai_templates_section_3',
        ['template_number' => 3]
    );
    add_settings_field(
        'btai_position_type_3',
        'Position Type',
        'btai_position_type_render',
        'btai_settings',
        'btai_templates_section_3',
        ['template_number' => 3]
    );
    add_settings_field(
        'btai_paragraph_number_3',
        'Insert After Paragraph #',
        'btai_paragraph_number_render',
        'btai_settings',
        'btai_templates_section_3',
        ['template_number' => 3]
    );
    add_settings_field(
        'btai_min_paragraphs_3',
        'Minimum Required Paragraphs',
        'btai_min_paragraphs_render',
        'btai_settings',
        'btai_templates_section_3',
        ['template_number' => 3]
    );

    // First Custom Content Section
    add_settings_section(
        'btai_custom_section_1',
        'First Custom Content',
        'btai_custom_section_1_callback',
        'btai_settings'
    );
    add_settings_field(
        'btai_custom_content_1',
        'Content',
        'btai_custom_content_render',
        'btai_settings',
        'btai_custom_section_1',
        ['content_number' => 1]
    );
    add_settings_field(
        'btai_custom_paragraph_1',
        'Insert After Paragraph #',
        'btai_custom_paragraph_render',
        'btai_settings',
        'btai_custom_section_1',
        ['content_number' => 1]
    );

    // Second Custom Content Section
    add_settings_section(
        'btai_custom_section_2',
        'Second Custom Content',
        'btai_custom_section_2_callback',
        'btai_settings'
    );
    add_settings_field(
        'btai_custom_content_2',
        'Content',
        'btai_custom_content_render',
        'btai_settings',
        'btai_custom_section_2',
        ['content_number' => 2]
    );
    add_settings_field(
        'btai_custom_paragraph_2',
        'Insert After Paragraph #',
        'btai_custom_paragraph_render',
        'btai_settings',
        'btai_custom_section_2',
        ['content_number' => 2]
    );

    // Categories Section
    add_settings_section(
        'btai_categories_section',
        'Category Settings',
        'btai_categories_section_callback',
        'btai_settings'
    );
    add_settings_field(
        'btai_categories',
        'Apply to Categories',
        'btai_categories_render',
        'btai_settings',
        'btai_categories_section'
    );
}

// New Position Type Renderer
function btai_position_type_render($args) {
    $options = get_option('btai_settings');
    $template_number = $args['template_number'];
    $field_name = $template_number === 1 ? 'position_type' : 'position_type_' . $template_number;
    $current_value = isset($options[$field_name]) ? $options[$field_name] : BTAI_POSITION_AFTER_PARAGRAPH;
    ?>
    <select name='btai_settings[<?php echo esc_attr($field_name); ?>]' class="btai-position-select">
        <option value="<?php echo BTAI_POSITION_AFTER_PARAGRAPH; ?>" <?php selected($current_value, BTAI_POSITION_AFTER_PARAGRAPH); ?>>
            After X paragraphs
        </option>
        <option value="<?php echo BTAI_POSITION_BEFORE_LAST; ?>" <?php selected($current_value, BTAI_POSITION_BEFORE_LAST); ?>>
            Before last paragraph
        </option>
        <option value="<?php echo BTAI_POSITION_AFTER_LAST; ?>" <?php selected($current_value, BTAI_POSITION_AFTER_LAST); ?>>
            After last paragraph
        </option>
    </select>
    <?php
}

// New Minimum Paragraphs Renderer
function btai_min_paragraphs_render($args) {
    $options = get_option('btai_settings');
    $template_number = $args['template_number'];
    $field_name = $template_number === 1 ? 'min_paragraphs' : 'min_paragraphs_' . $template_number;
    ?>
    <input type='number' min='1' name='btai_settings[<?php echo esc_attr($field_name); ?>]' 
           value='<?php echo isset($options[$field_name]) ? esc_attr($options[$field_name]) : ''; ?>'>
    <p class="description">Minimum number of paragraphs required in the post for this template to be inserted. Leave empty for no minimum.</p>
    <?php
}



function btai_template_dropdown_render($args) {
    $options = get_option('btai_settings');
    $template_number = $args['template_number'];
    $field_name = $template_number === 1 ? 'template_id' : 'template_id_' . $template_number;
    $templates = btai_get_bricks_templates();
    ?>
    <select name='btai_settings[<?php echo esc_attr($field_name); ?>]' class="btai-template-select">
        <option value=""><?php echo esc_html__('-- Select Template --', 'btai'); ?></option>
        <?php foreach ($templates as $id => $title) : ?>
            <option value="<?php echo esc_attr($id); ?>" <?php selected(isset($options[$field_name]) ? $options[$field_name] : '', $id); ?>>
                <?php echo esc_html($title); ?> (ID: <?php echo esc_html($id); ?>)
            </option>
        <?php endforeach; ?>
    </select>
    <?php if (empty($templates)) : ?>
        <p class="description" style="color: #d63638;">No Bricks templates found. Please create some templates first.</p>
    <?php endif; ?>
    <?php
}

// Section Callbacks
function btai_templates_section_1_callback() {
    echo '<p>Configure where to insert your first Bricks template in the post content.</p>';
}

function btai_templates_section_2_callback() {
    echo '<p>Configure where to insert your second Bricks template in the post content.</p>';
}

function btai_templates_section_3_callback() {
    echo '<p>Configure where to insert your third Bricks template in the post content.</p>';
}

function btai_custom_section_1_callback() {
    echo '<p>Add custom HTML or text content to be inserted in your post.</p>';
}

function btai_custom_section_2_callback() {
    echo '<p>Add additional custom HTML or text content (optional).</p>';
}

function btai_categories_section_callback() {
    echo '<p>Select which categories this content should appear in.</p>';
}

// Modular Template Field Renderers
function btai_template_id_render($args) {
    $options = get_option('btai_settings');
    $template_number = $args['template_number'];
    $field_name = $template_number === 1 ? 'template_id' : 'template_id_' . $template_number;
    ?>
    <input type='text' name='btai_settings[<?php echo esc_attr($field_name); ?>]' 
           value='<?php echo isset($options[$field_name]) ? esc_attr($options[$field_name]) : ''; ?>'>
    <p class="description">Enter the Bricks template ID</p>
    <?php
}

function btai_paragraph_number_render($args) {
    $options = get_option('btai_settings');
    $template_number = $args['template_number'];
    $field_name = $template_number === 1 ? 'paragraph_number' : 'paragraph_number_' . $template_number;
    ?>
    <input type='number' min='1' name='btai_settings[<?php echo esc_attr($field_name); ?>]' 
           value='<?php echo isset($options[$field_name]) ? esc_attr($options[$field_name]) : ''; ?>'>
    <?php
}

// Modular Custom Content Renderers
function btai_custom_content_render($args) {
    $options = get_option('btai_settings');
    $content_number = $args['content_number'];
    $field_name = 'custom_content_' . $content_number;
    ?>
    <textarea name='btai_settings[<?php echo esc_attr($field_name); ?>]' rows='5' style='width: 100%; max-width: 600px;'><?php 
        echo isset($options[$field_name]) ? esc_textarea($options[$field_name]) : ''; 
    ?></textarea>
    <p class="description">Enter custom HTML or text content to insert</p>
    <?php
}

function btai_custom_paragraph_render($args) {
    $options = get_option('btai_settings');
    $content_number = $args['content_number'];
    $field_name = 'custom_paragraph_' . $content_number;
    ?>
    <input type='number' min='1' name='btai_settings[<?php echo esc_attr($field_name); ?>]' 
           value='<?php echo isset($options[$field_name]) ? esc_attr($options[$field_name]) : ''; ?>'>
    <?php
}

function btai_categories_render() {
    $options = get_option('btai_settings');
    $categories = get_categories(['hide_empty' => false]);
    ?>
    <select multiple name='btai_settings[categories][]' style="min-width: 200px; min-height: 100px;">
        <?php foreach ($categories as $category): ?>
            <option value='<?php echo $category->term_id; ?>' 
                <?php echo (isset($options['categories']) && in_array($category->term_id, (array)$options['categories'])) ? 'selected' : ''; ?>>
                <?php echo $category->name; ?>
            </option>
        <?php endforeach; ?>
    </select>
    <p class="description">Hold Ctrl/Cmd to select multiple categories</p>
    <?php
}

function btai_options_page() {
    if (!current_user_can('manage_options')) {
        return;
    }
    ?>
    <div class="wrap bricks-admin-wrapper">
        <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
        
        <?php if (!class_exists('Bricks\Database')) : ?>
            <div class="notice notice-error">
                <p><?php _e('Bricks builder is not active. Please activate Bricks to use this plugin.', 'btai'); ?></p>
            </div>
        <?php endif; ?>

        <div class="bricks-admin-box" style="margin-top: 20px;">
            <form action='options.php' method='post'>
                <?php
                settings_fields('btai_settings');
                do_settings_sections('btai_settings');
                submit_button();
                ?>
            </form>
        </div>


        <!-- Hides the Insert After Paragraph # textfield when the position type is not set to After X paragraphs. -->
        <script>
            document.addEventListener('DOMContentLoaded', function() {
                function handleParagraphFields() {
                    const positionSelects = document.querySelectorAll('.btai-position-select');
                    
                    positionSelects.forEach(select => {
                        const row = select.closest('tr');
                        const nextRow = row.nextElementSibling; // Paragraph number row
                        
                        // Set initial state
                        if (select.value !== '<?php echo BTAI_POSITION_AFTER_PARAGRAPH; ?>') {
                            nextRow.style.display = 'none';
                        }
                        
                        // Add change handler
                        select.addEventListener('change', function() {
                            if (this.value === '<?php echo BTAI_POSITION_AFTER_PARAGRAPH; ?>') {
                                nextRow.style.display = 'table-row';
                            } else {
                                nextRow.style.display = 'none';
                            }
                        });
                    });
                }
                
                handleParagraphFields();
            });
        </script>

    </div>

    <style>
        .bricks-admin-wrapper {
            margin: 20px 20px 0 0;
        }
        .bricks-admin-box {
            background: #fff;
            border-radius: 3px;
            padding: 20px;
            box-shadow: 0 1px 1px rgba(0,0,0,.04);
        }
        .bricks-admin-box h2 {
            padding: 20px;
            margin: -20px -20px 20px;
            background: #f8f9fa;
            border-bottom: 1px solid #dee2e6;
            color: #1d2327;
            font-size: 1.2em;
            font-weight: 600;
        }
        .bricks-admin-box select,
        .bricks-admin-box input[type="text"],
        .bricks-admin-box input[type="number"] {
            min-height: 36px;
            border-radius: 3px;
            border: 1px solid #ddd;
            padding: 0 8px;
            width: 100%;
            max-width: 25em;
        }
        .btai-template-select {
            min-width: 300px;
        }
        .bricks-admin-box .form-table {
            margin-bottom: 30px;
        }
        .bricks-admin-box .form-table th {
            padding: 20px 10px 20px 0;
            width: 200px;
        }
        .bricks-admin-box .description {
            color: #666;
            font-style: italic;
            margin-top: 4px;
        }
        .bricks-admin-box textarea {
            border: 1px solid #ddd;
            border-radius: 3px;
            padding: 8px;
        }
    </style>
    <?php
}


// Content filter to insert templates
add_filter('the_content', 'btai_insert_templates', 20);

function btai_insert_templates($content) {
    if (!is_single() || btai_is_excerpt()) {
        return $content;
    }

    $options = get_option('btai_settings');
    if (empty($options)) {
        return $content;
    }

    // Check categories
    if (!empty($options['categories'])) {
        $post_categories = wp_get_post_categories(get_the_ID());
        $has_matching_category = false;
        foreach ($post_categories as $category) {
            if (in_array($category, (array)$options['categories'])) {
                $has_matching_category = true;
                break;
            }
        }
        if (!$has_matching_category) {
            return $content;
        }
    }

    // Count total paragraphs once
    $paragraphs = explode('</p>', $content);
    $total_paragraphs = count(array_filter($paragraphs, 'trim'));

    // Create arrays for different insertion points
    $insertions = [];
    $after_content = [];
    $before_last = [];
    
    // Process Bricks templates
    for ($i = 1; $i <= 3; $i++) {
        $template_id = $i === 1 ? 'template_id' : 'template_id_' . $i;
        $position_type = $i === 1 ? 'position_type' : 'position_type_' . $i;
        $paragraph_number = $i === 1 ? 'paragraph_number' : 'paragraph_number_' . $i;
        $min_paragraphs = $i === 1 ? 'min_paragraphs' : 'min_paragraphs_' . $i;

        // Skip if template not selected or minimum paragraphs not met
        if (empty($options[$template_id]) || 
            (!empty($options[$min_paragraphs]) && $total_paragraphs < intval($options[$min_paragraphs]))) {
            continue;
        }

        $template_content = do_shortcode('[bricks_template id="' . esc_attr($options[$template_id]) . '"]');

        switch ($options[$position_type]) {
            case BTAI_POSITION_AFTER_PARAGRAPH:
                if (!empty($options[$paragraph_number])) {
                    $insertions[$options[$paragraph_number]][] = $template_content;
                }
                break;
            case BTAI_POSITION_BEFORE_LAST:
                $before_last[] = $template_content;
                break;
            case BTAI_POSITION_AFTER_LAST:
                $after_content[] = $template_content;
                break;
        }
    }
    
    // Process custom content sections
    for ($i = 1; $i <= 2; $i++) {
        $content_key = 'custom_content_' . $i;
        $position_type = 'custom_position_type_' . $i;
        $paragraph_number = 'custom_paragraph_' . $i;
        $min_paragraphs = 'custom_min_paragraphs_' . $i;

        // Skip if content not set or minimum paragraphs not met
        if (empty($options[$content_key]) || 
            (!empty($options[$min_paragraphs]) && $total_paragraphs < intval($options[$min_paragraphs]))) {
            continue;
        }

        $custom_content = wp_kses_post($options[$content_key]);

        switch ($options[$position_type]) {
            case BTAI_POSITION_AFTER_PARAGRAPH:
                if (!empty($options[$paragraph_number])) {
                    $insertions[$options[$paragraph_number]][] = $custom_content;
                }
                break;
            case BTAI_POSITION_BEFORE_LAST:
                $before_last[] = $custom_content;
                break;
            case BTAI_POSITION_AFTER_LAST:
                $after_content[] = $custom_content;
                break;
        }
    }

    // Build the new content
    $new_content = '';
    $p_count = 0;

    foreach ($paragraphs as $paragraph) {
        if (trim($paragraph)) {
            $p_count++;

            // Insert content before last paragraph
            if ($p_count === $total_paragraphs && !empty($before_last)) {
                $new_content .= implode('', $before_last);
            }

            $new_content .= $paragraph . '</p>';

            // Insert content after specific paragraphs
            if (isset($insertions[$p_count])) {
                $new_content .= implode('', $insertions[$p_count]);
            }
        }
    }

    // Add content after the last paragraph
    if (!empty($after_content)) {
        $new_content .= implode('', $after_content);
    }

    return $new_content;
}


// Helper function to check if we're in an excerpt
function btai_is_excerpt() {
    global $wp_current_filter;
    return in_array('get_the_excerpt', $wp_current_filter);
}

// Add the filter for excerpts to prevent template insertion
add_filter('get_the_excerpt', function($excerpt) {
    // Remove our content filter while getting the excerpt
    remove_filter('the_content', 'btai_insert_templates', 20);
    $excerpt = wp_trim_excerpt($excerpt);
    // Add the filter back
    add_filter('the_content', 'btai_insert_templates', 20);
    return $excerpt;
}, 9);

