<?php
if (!defined('ABSPATH')) exit;

if (!class_exists('BGS_Query_Element')) {
    class BGS_Query_Element extends \Bricks\Element {
        public $category = 'google sheets';
        public $name = 'bgs-sheet-1';
        public $icon = 'ti-layout-grid2';
        public $css_selector = '.bgs-query';
        public $scripts = ['bgsQueryElement'];
        public $nestable = true;
        public $loop = true;

        public function get_label() {
            return esc_html__('Google Sheets', 'bricks');
        }
        
        public function get_keywords() {
            return ['google', 'sheets', 'data', 'query', 'loop'];
        }

        public function set_control_groups() {
            $this->control_groups['query'] = [
                'title' => esc_html__('Query', 'bricks'),
                'tab' => 'content',
            ];

            $this->control_groups['filters'] = [
                'title' => esc_html__('Filters', 'bricks'),
                'tab' => 'content',
            ];
        
            $this->control_groups['grouping'] = [
                'title' => esc_html__('Grouping', 'bricks'),
                'tab' => 'content',
            ];

            $this->control_groups['pagination'] = [
                'title' => esc_html__('Pagination', 'bricks'),
                'tab' => 'content',
            ];
        }

        public function set_controls() {
            // Query Settings
            $this->controls['queryLimit'] = [
                'tab' => 'content',
                'group' => 'query',
                'label' => esc_html__('Items per page', 'bricks'),
                'type' => 'number',
                'min' => 1,
                'default' => 10,
            ];

            $this->controls['queryOffset'] = [
                'tab' => 'content',
                'group' => 'query',
                'label' => esc_html__('Offset', 'bricks'),
                'type' => 'number',
                'min' => 0,
                'default' => 0,
            ];

            // Filter Controls
            $this->controls['filters'] = [
                'tab' => 'content',
                'group' => 'filters',
                'label' => esc_html__('Filter conditions', 'bricks'),
                'type' => 'repeater',
                'titleProperty' => 'filterColumn', // Shows selected column as repeater item title
                'placeholder' => esc_html__('Filter', 'bricks'),
                'fields' => [
                    'filterColumn' => [
                        'label' => esc_html__('Column', 'bricks'),
                        'type' => 'select',
                        'options' => $this->get_sheet_columns(),
                        'inline' => true,
                    ],
                    'filterOperator' => [
                        'label' => esc_html__('Operator', 'bricks'),
                        'type' => 'select',
                        'options' => [
                            'contains' => 'Contains',
                            'equals' => 'Equals',
                            'starts' => 'Starts with',
                            'ends' => 'Ends with',
                            'greater' => 'Greater than',
                            'less' => 'Less than',
                            'between' => 'Between',
                            'not_empty' => 'Not empty',
                        ],
                        'default' => 'contains',
                        'inline' => true,
                    ],
                    'filterValueType' => [
                        'label' => esc_html__('Value type', 'bricks'),
                        'type' => 'select',
                        'options' => [
                            'static' => esc_html__('Static Value', 'bricks'),
                            'custom_field' => esc_html__('Custom Field', 'bricks'),
                        ],
                        'default' => 'static',
                        'inline' => true,
                    ],
                    'filterValue' => [
                        'label' => esc_html__('Value', 'bricks'),
                        'type' => 'text',
                        'conditions' => [
                            'filterValueType' => 'static',
                        ],
                        'inline' => true,
                    ],
                    'filterCustomField' => [
                        'label' => esc_html__('Custom field', 'bricks'),
                        'type' => 'text',
                        'conditions' => [
                            'filterValueType' => 'custom_field',
                        ],
                        'inline' => true,
                    ],
                ],
            ];

             // Ordering Controls
             $this->controls['orderBy'] = [
                'tab' => 'content',
                'group' => 'query',
                'label' => esc_html__('Order by', 'bricks'),
                'type' => 'select',
                'options' => $this->get_sheet_columns(),
                'clearable' => true,
                'placeholder' => 'Select column to sort by',
                'inline' => true,
            ];

            $this->controls['orderDir'] = [
                'tab' => 'content',
                'group' => 'query',
                'label' => esc_html__('Order', 'bricks'),
                'type' => 'select',
                'options' => [
                    'asc' => 'Ascending',
                    'desc' => 'Descending'
                ],
                'default' => 'asc',
                'required' => ['orderBy', '!=', ''],
                'inline' => true,
            ];

            // Date Range Controls
            $this->controls['dateColumn'] = [
                'tab' => 'content',
                'group' => 'query',
                'label' => esc_html__('Date Column', 'bricks'),
                'type' => 'select',
                'options' => $this->get_sheet_columns(),
                'clearable' => true,
                'placeholder' => 'Select date column',
                'inline' => true,
            ];

            $this->controls['dateRangeType'] = [
                'tab' => 'content',
                'group' => 'query',
                'label' => esc_html__('Date Range', 'bricks'),
                'type' => 'select',
                'options' => [
                    'today' => 'Today',
                    'yesterday' => 'Yesterday',
                    'this_week' => 'This Week',
                    'last_week' => 'Last Week',
                    'this_month' => 'This Month',
                    'last_month' => 'Last Month',
                    'custom' => 'Custom Range',
                    'upcoming' => 'Upcoming',
                    'past' => 'Past'
                ],
                'default' => 'custom',
                'required' => ['dateColumn', '!=', ''],
            ];

            $this->controls['customDateStart'] = [
                'tab' => 'content',
                'group' => 'query',
                'label' => esc_html__('Start Date', 'bricks'),
                'type' => 'date',
                'required' => [['dateColumn', '!=', ''], ['dateRangeType', '=', 'custom']],
            ];

            $this->controls['customDateEnd'] = [
                'tab' => 'content',
                'group' => 'query',
                'label' => esc_html__('End Date', 'bricks'),
                'type' => 'date',
                'required' => [['dateColumn', '!=', ''], ['dateRangeType', '=', 'custom']],
            ];

            // Add these new controls in set_controls() method after existing controls
            $this->controls['enableGrouping'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Enable grouping', 'bricks'),
                'type' => 'checkbox',
            ];

            $this->controls['groupHeaderPrefix'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Header Prefix', 'bricks'),
                'type' => 'text',
                'inline' => true,
                'required' => ['enableGrouping', '=', true],
            ];
            
            $this->controls['groupHeaderSuffix'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Header Suffix', 'bricks'),
                'type' => 'text',
                'inline' => true,
                'required' => ['enableGrouping', '=', true],
            ];
            
            $this->controls['showGroupCount'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Show Item Count', 'bricks'),
                'type' => 'checkbox',
                'required' => ['enableGrouping', '=', true],
            ];
            
            $this->controls['groupSortDir'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Group Sort Direction', 'bricks'),
                'type' => 'select',
                'options' => [
                    'asc' => 'Ascending',
                    'desc' => 'Descending'
                ],
                'default' => 'asc',
                'required' => ['enableGrouping', '=', true],
            ];
            

            $this->controls['groupByColumn'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Group by column', 'bricks'),
                'type' => 'select',
                'options' => $this->get_sheet_columns(),
                'required' => ['enableGrouping', '=', true],
            ];

            $this->controls['groupHeaderTypography'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Header typography', 'bricks'),
                'type' => 'typography',
                'css' => [
                    [
                        'property' => 'typography',
                        'selector' => '.bgs-group-header',
                    ],
                ],
                'required' => ['enableGrouping', '=', true],
            ];

            $this->controls['groupHeaderBackground'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Header background', 'bricks'),
                'type' => 'color',
                'css' => [
                    [
                        'property' => 'background-color',
                        'selector' => '.bgs-group-header',
                    ],
                ],
                'required' => ['enableGrouping', '=', true],
            ];

            $this->controls['groupHeaderPadding'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Header padding', 'bricks'),
                'type' => 'dimensions',
                'css' => [
                    [
                        'property' => 'padding',
                        'selector' => '.bgs-group-header',
                    ],
                ],
                'required' => ['enableGrouping', '=', true],
            ];

            $this->controls['groupHeaderMargin'] = [
                'tab' => 'content',
                'group' => 'grouping',
                'label' => esc_html__('Header margin', 'bricks'),
                'type' => 'dimensions',
                'css' => [
                    [
                        'property' => 'margin',
                        'selector' => '.bgs-group-header',
                    ],
                ],
                'required' => ['enableGrouping', '=', true],
            ];

            // Pagination Controls
            $this->controls['pagination'] = [
                'tab' => 'content',
                'group' => 'pagination',
                'label' => esc_html__('Enable pagination', 'bricks'),
                'type' => 'checkbox',
            ];

            $this->controls['paginationType'] = [
                'tab' => 'content',
                'group' => 'pagination',
                'label' => esc_html__('Pagination type', 'bricks'),
                'type' => 'select',
                'options' => [
                    'numbered' => esc_html__('Numbered', 'bricks'),
                    'simple' => esc_html__('Simple', 'bricks'),
                ],
                'default' => 'numbered',
                'required' => ['pagination', '=', true],
            ];
        }

        public function get_nestable_children() {
            return [
                [
                    'name' => 'block',
                    'label' => esc_html__('Sheet Item', 'bricks'),
                ]
            ];
        }

        private function get_sheet_columns() {
            static $columns = null;
            
            if ($columns !== null) {
                return $columns;
            }
            
            // Check if function exists
            if (!function_exists('bgs_get_sheet1_data')) {
                return [];
            }
            
            $sheet_data = bgs_get_sheet1_data(); // Change this function call
            $columns = [];
            
            if (!is_wp_error($sheet_data) && !empty($sheet_data)) {
                $first_row = reset($sheet_data);
                if ($first_row) {
                    $columns = array_combine(
                        array_map('sanitize_title', array_keys($first_row)),
                        array_keys($first_row)
                    );
                }
            }
            
            return $columns;
        }

        private function get_filtered_data() {
            $settings = $this->settings;
            
            // Get sheet data
            $sheet_data = bgs_get_sheet1_data();
            
            if (is_wp_error($sheet_data) || empty($sheet_data)) {
                return [];
            }
        
            $filtered_data = $sheet_data;
        
            // Apply filters if set
            if (!empty($settings['filters']) && is_array($settings['filters'])) {
                foreach ($settings['filters'] as $filter) {
                    if (empty($filter['filterColumn']) || empty($filter['filterOperator'])) {
                        continue;
                    }
        
                    // Get filter value based on value type
                    $filter_value = '';
                    if ($filter['filterValueType'] === 'static' && isset($filter['filterValue'])) {
                        $filter_value = $filter['filterValue'];
                    } elseif ($filter['filterValueType'] === 'custom_field' && !empty($filter['filterCustomField'])) {
                        // Get custom field value
                        $custom_field_tag = trim($filter['filterCustomField'], '{}');
                        if (strpos($custom_field_tag, 'cf_') === 0) {
                            // Remove 'cf_' prefix if present
                            $custom_field_name = substr($custom_field_tag, 3);
                        } else {
                            $custom_field_name = $custom_field_tag;
                        }
                        
                        // Get the current post ID
                        $post_id = get_the_ID();
                        if (!$post_id) {
                            global $post;
                            $post_id = $post ? $post->ID : 0;
                        }
                        
                        if ($post_id) {
                            $filter_value = get_post_meta($post_id, $custom_field_name, true);
                        }
                    }
        
                    // Skip this filter if no value to filter by
                    if ($filter_value === '') {
                        continue;
                    }
        
                    $filtered_data = array_filter($filtered_data, function($row) use ($filter, $filter_value) {
                        if (!isset($row[$filter['filterColumn']]) && !array_key_exists($filter['filterColumn'], $row)) {
                            // Try to find the column by comparing sanitized names
                            foreach ($row as $header => $value) {
                                if (sanitize_title($header) === $filter['filterColumn']) {
                                    $column_value = $value;
                                    break;
                                }
                            }
                            if (!isset($column_value)) {
                                return false;
                            }
                        } else {
                            $column_value = $row[$filter['filterColumn']];
                        }
        
                        // Case-insensitive comparison
                        $column_value = strtolower(trim((string)$column_value));
                        $filter_value = strtolower(trim((string)$filter_value));
        
                        switch ($filter['filterOperator']) {
                            case 'contains':
                                return str_contains($column_value, $filter_value);
                            case 'equals':
                                return $column_value === $filter_value;
                            case 'starts':
                                return str_starts_with($column_value, $filter_value);
                            case 'ends':
                                return str_ends_with($column_value, $filter_value);
                            case 'greater':
                                return is_numeric($column_value) && is_numeric($filter_value) && 
                                       floatval($column_value) > floatval($filter_value);
                            case 'less':
                                return is_numeric($column_value) && is_numeric($filter_value) && 
                                       floatval($column_value) < floatval($filter_value);
                            case 'between':
                                if (!is_numeric($column_value)) return false;
                                $range = explode(',', $filter_value);
                                if (count($range) !== 2) return false;
                                return floatval($column_value) >= floatval(trim($range[0])) && 
                                       floatval($column_value) <= floatval(trim($range[1]));
                            case 'not_empty':
                                return !empty($column_value);
                            default:
                                return false;
                        }
                    });
                }
            }
        
            // Apply date range filtering if set
            if (!empty($settings['dateColumn'])) {
                $date_column = $settings['dateColumn'];
                $range_type = !empty($settings['dateRangeType']) ? $settings['dateRangeType'] : 'custom';
                
                // Calculate date range based on type
                $start_date = null;
                $end_date = null;
                
                switch ($range_type) {
                    case 'today':
                        $start_date = strtotime('today');
                        $end_date = strtotime('tomorrow') - 1;
                        break;
                    case 'yesterday':
                        $start_date = strtotime('yesterday');
                        $end_date = strtotime('today') - 1;
                        break;
                    case 'this_week':
                        $start_date = strtotime('this week Monday');
                        $end_date = strtotime('this week Sunday') + 86399;
                        break;
                    case 'last_week':
                        $start_date = strtotime('last week Monday');
                        $end_date = strtotime('last week Sunday') + 86399;
                        break;
                    case 'this_month':
                        $start_date = strtotime('first day of this month');
                        $end_date = strtotime('last day of this month') + 86399;
                        break;
                    case 'last_month':
                        $start_date = strtotime('first day of last month');
                        $end_date = strtotime('last day of last month') + 86399;
                        break;
                    case 'custom':
                        if (!empty($settings['customDateStart'])) {
                            $start_date = strtotime($settings['customDateStart']);
                        }
                        if (!empty($settings['customDateEnd'])) {
                            $end_date = strtotime($settings['customDateEnd']) + 86399;
                        }
                        break;
                    case 'upcoming':
                        $start_date = strtotime('today');
                        $end_date = PHP_INT_MAX;
                        break;
                    case 'past':
                        $start_date = 0;
                        $end_date = strtotime('yesterday') + 86399;
                        break;
                }
        
                if ($start_date !== null || $end_date !== null) {
                    $filtered_data = array_filter($filtered_data, function($row) use ($date_column, $start_date, $end_date) {
                        if (!isset($row[$date_column])) {
                            return false;
                        }
        
                        $row_date = strtotime($row[$date_column]);
                        if (!$row_date) {
                            return false;
                        }
        
                        if ($start_date !== null && $row_date < $start_date) {
                            return false;
                        }
        
                        if ($end_date !== null && $row_date > $end_date) {
                            return false;
                        }
        
                        return true;
                    });
                }
            }
        
            // Apply ordering if set
            if (!empty($settings['orderBy'])) {
                $order_column = $settings['orderBy'];
                $order_direction = isset($settings['orderDir']) ? strtolower($settings['orderDir']) : 'asc';
                
                // Find the actual column name if it's sanitized
                $actual_column = $order_column;
                if (!empty($filtered_data)) {
                    $first_row = reset($filtered_data);
                    foreach (array_keys($first_row) as $column) {
                        if (sanitize_title($column) === $order_column) {
                            $actual_column = $column;
                            break;
                        }
                    }
                }
        
                usort($filtered_data, function($a, $b) use ($actual_column, $order_direction) {
                    $a_val = $a[$actual_column] ?? '';
                    $b_val = $b[$actual_column] ?? '';
        
                    // Skip comparison if either value is empty
                    if ($a_val === '' && $b_val === '') return 0;
                    if ($a_val === '') return $order_direction === 'asc' ? -1 : 1;
                    if ($b_val === '') return $order_direction === 'asc' ? 1 : -1;
        
                    // Try to parse as date first
                    $a_date = strtotime($a_val);
                    $b_date = strtotime($b_val);
        
                    if ($a_date && $b_date) {
                        // Date comparison
                        return $order_direction === 'asc' ? 
                            ($a_date - $b_date) : 
                            ($b_date - $a_date);
                    }
        
                    // Check if values are numeric
                    $is_numeric = is_numeric($a_val) && is_numeric($b_val);
                    
                    if ($is_numeric) {
                        // Convert to float for numeric comparison
                        $a_val = floatval($a_val);
                        $b_val = floatval($b_val);
                        return $order_direction === 'asc' ? 
                            ($a_val - $b_val) : 
                            ($b_val - $a_val);
                    }
        
                    // Default to string comparison
                    $a_val = strtolower(strval($a_val));
                    $b_val = strtolower(strval($b_val));
                    
                    $comparison = strcmp($a_val, $b_val);
                    return $order_direction === 'asc' ? $comparison : -$comparison;
                });
            }
        
            return array_values($filtered_data);
        }

        public function render() {
            $settings = $this->settings;
            $filtered_data = $this->get_filtered_data();
        
            // Root classes
            $root_classes = ['bgs-sheets-query'];
            $this->set_attribute('_root', 'class', $root_classes);
        
            // Pagination settings
            $items_per_page = !empty($settings['queryLimit']) ? intval($settings['queryLimit']) : 10;
            $offset = !empty($settings['queryOffset']) ? intval($settings['queryOffset']) : 0;
            $current_page = max(1, get_query_var('paged'));
        
            // Calculate pagination
            $total_items = count($filtered_data);
            $total_pages = ceil($total_items / $items_per_page);
            $start = ($current_page - 1) * $items_per_page + $offset;
            
            // Get items for current page
            $items = array_slice($filtered_data, $start, $items_per_page);
        
            echo "<div {$this->render_attributes('_root')}>";
        
            if (!empty($items)) {
                echo "<div class='bgs-items'>";
                
                // Check if grouping is enabled
                if (!empty($settings['enableGrouping']) && !empty($settings['groupByColumn'])) {
                    // Group the items
                    $grouped_items = [];
                    foreach ($items as $item) {
                        // Find the actual column name if using sanitized version
                        $group_column = $settings['groupByColumn'];
                        $actual_column = $group_column;
                        
                        // Find the actual column name by comparing sanitized versions
                        foreach ($item as $col => $val) {
                            if (sanitize_title($col) === $group_column) {
                                $actual_column = $col;
                                break;
                            }
                        }
        
                        // Get group value, default to 'Ungrouped' if empty
                        $group_value = isset($item[$actual_column]) && $item[$actual_column] !== '' 
                            ? $item[$actual_column] 
                            : 'Ungrouped';
        
                        if (!isset($grouped_items[$group_value])) {
                            $grouped_items[$group_value] = [];
                        }
                        $grouped_items[$group_value][] = $item;
                    }
        
                    // Sort group headers if needed
                    if (!empty($settings['groupSortDir'])) {
                        $sort_dir = $settings['groupSortDir'];
                        if ($sort_dir === 'asc') {
                            ksort($grouped_items, SORT_NATURAL | SORT_FLAG_CASE);
                        } else {
                            krsort($grouped_items, SORT_NATURAL | SORT_FLAG_CASE);
                        }
                    }
        
                    // Store the query data globally
                    $GLOBALS['bgs_current_query'] = [
                        'provider' => 'google_sheets',
                        'current_page' => $current_page,
                        'count' => count($items),
                        'total_count' => $total_items,
                        'total_pages' => $total_pages,
                        'limit' => $items_per_page,
                        'offset' => $offset,
                        'items' => $items,
                        'current_index' => 0,
                        'is_grouped' => true,
                        'groups' => array_keys($grouped_items)
                    ];
        
                    // Initialize group counter
                    $group_index = 0;
        
                    // Render grouped items
                    foreach ($grouped_items as $group_value => $group_items) {
                        // Store current group info in globals
                        $GLOBALS['bgs_current_group'] = [
                            'value' => $group_value,
                            'index' => $group_index,
                            'total' => count($grouped_items),
                            'items_count' => count($group_items)
                        ];
        
                        echo "<div class='bgs-group' data-group-index='{$group_index}'>";
                        
                        // Group header with optional prefix/suffix
                        $header_prefix = isset($settings['groupHeaderPrefix']) ? $settings['groupHeaderPrefix'] : '';
                        $header_suffix = isset($settings['groupHeaderSuffix']) ? $settings['groupHeaderSuffix'] : '';
                        
                        echo "<div class='bgs-group-header'>";
                        if ($header_prefix) {
                            echo "<span class='group-header-prefix'>" . esc_html($header_prefix) . "</span>";
                        }
                        echo "<span class='group-header-value'>" . esc_html($group_value) . "</span>";
                        if ($header_suffix) {
                            echo "<span class='group-header-suffix'>" . esc_html($header_suffix) . "</span>";
                        }
                        if (!empty($settings['showGroupCount'])) {
                            echo "<span class='group-header-count'>(" . count($group_items) . ")</span>";
                        }
                        echo "</div>";
        
                        echo "<div class='bgs-group-items'>";
        
                        // Render items in this group
                        foreach ($group_items as $index => $item) {
                            $GLOBALS['bgs_current_item'] = $item;
                            $GLOBALS['bgs_current_index'] = $index;
                            $GLOBALS['bgs_current_query']['current_index'] = $index;
                            $GLOBALS['bgs_current_query']['in_group'] = true;
                            $GLOBALS['bgs_current_query']['group_index'] = $group_index;
        
                            echo "<div class='bgs-item' data-item-index='{$index}'>";
                            echo \Bricks\Frontend::render_children($this);
                            echo "</div>";
                        }
        
                        echo "</div>"; // Close group items
                        echo "</div>"; // Close group
        
                        $group_index++;
                    }
        
                    // Clean up group-specific globals
                    unset($GLOBALS['bgs_current_group']);
        
                } else {
                    // Regular non-grouped rendering
                    $GLOBALS['bgs_current_query'] = [
                        'provider' => 'google_sheets',
                        'current_page' => $current_page,
                        'count' => count($items),
                        'total_count' => $total_items,
                        'total_pages' => $total_pages,
                        'limit' => $items_per_page,
                        'offset' => $offset,
                        'items' => $items,
                        'current_index' => 0,
                        'is_grouped' => false
                    ];
        
                    foreach ($items as $index => $item) {
                        $GLOBALS['bgs_current_item'] = $item;
                        $GLOBALS['bgs_current_index'] = $index;
                        $GLOBALS['bgs_current_query']['current_index'] = $index;
        
                        echo "<div class='bgs-item' data-item-index='{$index}'>";
                        echo \Bricks\Frontend::render_children($this);
                        echo "</div>";
                    }
                }
        
                // Clean up globals
                unset($GLOBALS['bgs_current_item']);
                unset($GLOBALS['bgs_current_index']);
                unset($GLOBALS['bgs_current_query']);
        
                echo "</div>"; // Close bgs-items
        
                // Render pagination if enabled
                if (!empty($settings['pagination']) && $total_pages > 1) {
                    $this->render_pagination($total_pages, $current_page);
                }
            } else {
                echo "<div class='bgs-no-items'>" . esc_html__('No items found', 'bricks') . "</div>";
            }
        
            echo "</div>";
        }
        
        

        private function render_pagination($total_pages, $current_page) {
            $settings = $this->settings;
            $pagination_type = !empty($settings['paginationType']) ? $settings['paginationType'] : 'numbered';

            echo "<div class='bgs-pagination'>";
            
            if ($pagination_type === 'numbered') {
                // Previous page
                if ($current_page > 1) {
                    echo "<a href='" . get_pagenum_link($current_page - 1) . "' class='bgs-prev'>←</a>";
                }

                // Page numbers
                for ($i = 1; $i <= $total_pages; $i++) {
                    $active_class = $current_page === $i ? ' active' : '';
                    echo "<a href='" . get_pagenum_link($i) . "' class='bgs-page{$active_class}'>{$i}</a>";
                }

                // Next page
                if ($current_page < $total_pages) {
                    echo "<a href='" . get_pagenum_link($current_page + 1) . "' class='bgs-next'>→</a>";
                }
            } else {
                // Simple pagination
                if ($current_page > 1) {
                    echo "<a href='" . get_pagenum_link($current_page - 1) . "' class='bgs-prev'>" . esc_html__('Previous', 'bricks') . "</a>";
                }

                if ($current_page < $total_pages) {
                    echo "<a href='" . get_pagenum_link($current_page + 1) . "' class='bgs-next'>" . esc_html__('Next', 'bricks') . "</a>";
                }
            }
            
            echo "</div>";
        }
    }
}