<?php
if (!defined('ABSPATH')) exit;

// Constants specific to this sheet
if (!defined('BGS1_CACHE_KEY')) {
    define('BGS1_CACHE_KEY', 'bgs_sheet1_data_cache');
}

if (!defined('BGS1_CACHE_EXPIRATION_KEY')) {
    define('BGS1_CACHE_EXPIRATION_KEY', 'bgs_sheet1_expiration');
}

if (!function_exists('bgs_get_sheet1_data')) {
    function bgs_get_sheet1_data() {
        $options = get_option('bgs_settings');
        
        // Check if we have cached data
        $cached_data = bgs_get_cached_data(BGS1_CACHE_KEY, BGS1_CACHE_EXPIRATION_KEY);
        if ($cached_data !== null) {
            return $cached_data;
        }
        
        // Fetch new data
        $data = bgs_fetch_sheet_data('bgs_settings', $options['sheet_id'] ?? '');
        
        if (!is_wp_error($data)) {
            // Apply column filtering before caching
            $filtered_data = bgs_filter_sheet_data($data, 1);
            
            // Get cache duration from settings, with fallback
            $cache_duration = isset($options['cache_duration']) ? intval($options['cache_duration']) : 60;
            
            // Cache the filtered data
            bgs_set_cached_data(
                $filtered_data, 
                BGS1_CACHE_KEY, 
                BGS1_CACHE_EXPIRATION_KEY,
                'bgs_settings',
                'bgs_last_sync_time',
                $cache_duration
            );
            
            return $filtered_data;
        }
        
        return $data;
    }
}

if (!function_exists('bgs_register_sheet1_handlers')) {
    function bgs_register_sheet1_handlers() {
        // Register dynamic tags
        add_filter('bricks/dynamic_tags_list', 'bgs_register_sheet1_tags');
        add_filter('bricks/dynamic_data/render_tag', 'bgs_render_sheet1_tag', 10, 3);
        
        // Register provider
        add_filter('bricks/query/register_providers', 'bgs_register_sheet1_provider');
        add_filter('bricks/dynamic_data/providers', 'bgs_register_sheet1_provider');
    }
}

if (!function_exists('bgs_register_sheet1_provider')) {
    function bgs_register_sheet1_provider($providers) {
        return bgs_register_base_provider(
            $providers,
            'bgs_settings',
            'bgs_get_sheet1_data',
            'google_sheets',
            'Google Sheets'
        );
    }
}

if (!function_exists('bgs_register_sheet1_dynamic_tags')) {
    function bgs_register_sheet1_dynamic_tags($tags) {
        return bgs_register_base_dynamic_tags(
            $tags,
            'bgs_get_sheet1_data',
            'sheets',
            'Google Sheets'
        );
    }
}

if (!function_exists('bgs_register_sheet1_tags')) {
    function bgs_register_sheet1_tags($tags) {
        return bgs_register_base_dynamic_tags(
            $tags,
            'bgs_get_sheet1_data',
            'sheets',
            'Google Sheets'
        );
    }
}

if (!function_exists('bgs_render_sheet1_tag')) {
    function bgs_render_sheet1_tag($tag, $post = null, $context = 'text') {
        // Handle case where $tag is an array
        if (is_array($tag)) {
            return $tag;
        }

        // Ensure $tag is a string
        $tag = (string)$tag;

        if (strpos($tag, '{sheets_') !== 0) {
            return $tag;
        }

        // Get the current query data
        $query = isset($GLOBALS['bgs_current_query']) ? $GLOBALS['bgs_current_query'] : null;
        $current_item = isset($GLOBALS['bgs_current_item']) ? $GLOBALS['bgs_current_item'] : null;
        
        // If no current item but we have query data, get the first item
        if (!$current_item && $query && !empty($query['items'])) {
            $current_item = reset($query['items']);
        }

        // If still no item, get the first row from all data
        if (!$current_item) {
            $sheet_data = bgs_get_sheet1_data();
            if (is_wp_error($sheet_data) || empty($sheet_data)) {
                return 'No data available';
            }
            $current_item = reset($sheet_data);
        }

        // Remove tag prefix and curly braces
        $tag_name = str_replace(['{sheets_', '}'], '', $tag);
        
        // Extract the column name from the tag
        foreach ($current_item as $column => $value) {
            if (sanitize_title($column) === $tag_name) {
                return $value;
            }
        }

        return $tag;
    }
}

if (!function_exists('bgs_render_sheet1_content')) {
    function bgs_render_sheet1_content($content, $post = null, $context = 'text') {
        if (strpos($content, '{sheets_') === false) {
            return $content;
        }

        preg_match_all('/{sheets_[^}]+}/', $content, $matches);
        foreach ($matches[0] as $tag) {
            $replacement = bgs_render_sheet1_tag($tag, $post, $context);
            $content = str_replace($tag, $replacement, $content);
        }

        return $content;
    }
}