<?php
/**
 * Add configurable webhook functionality to Bricks Builder forms
 */
function add_bricks_form_webhook() {
    // Add webhook settings to form
    add_filter('bricks/form/settings', function($settings) {
        $settings['webhook'] = [
            'tab' => 'actions',
            'group' => 'webhook',
            'label' => esc_html__('Webhooks', 'bricks'),
            'fields' => [
                'webhooks' => [
                    'label' => esc_html__('Webhook Configurations', 'bricks'),
                    'type' => 'repeater',
                    'fields' => [
                        'url' => [
                            'label' => esc_html__('Webhook URL', 'bricks'),
                            'type' => 'text',
                            'required' => true,
                        ],
                        'method' => [
                            'label' => esc_html__('HTTP Method', 'bricks'),
                            'type' => 'select',
                            'options' => [
                                'POST' => 'POST',
                                'GET' => 'GET',
                                'PUT' => 'PUT',
                                'PATCH' => 'PATCH',
                            ],
                            'default' => 'POST',
                        ],
                        'headers' => [
                            'label' => esc_html__('Custom Headers', 'bricks'),
                            'type' => 'repeater',
                            'fields' => [
                                'key' => [
                                    'label' => esc_html__('Header Key', 'bricks'),
                                    'type' => 'text',
                                ],
                                'value' => [
                                    'label' => esc_html__('Header Value', 'bricks'),
                                    'type' => 'text',
                                ],
                            ],
                        ],
                    ],
                ],
            ],
        ];
        return $settings;
    });

    // Send webhooks on form submission
    add_filter('bricks/form/submit', function($return, $form, $form_settings) {
        if (isset($form_settings['webhooks']) && is_array($form_settings['webhooks'])) {
            foreach ($form_settings['webhooks'] as $webhook) {
                if (empty($webhook['url'])) continue;

                $headers = ['Content-Type' => 'application/json'];
                if (!empty($webhook['headers']) && is_array($webhook['headers'])) {
                    foreach ($webhook['headers'] as $header) {
                        if (!empty($header['key']) && !empty($header['value'])) {
                            $headers[$header['key']] = $header['value'];
                        }
                    }
                }

                $args = [
                    'body' => json_encode($form['form_fields']),
                    'headers' => $headers,
                    'method' => $webhook['method'] ?? 'POST',
                ];

                $response = wp_remote_request($webhook['url'], $args);

                if (is_wp_error($response)) {
                    error_log('Bricks Form Webhook error: ' . $response->get_error_message());
                }
            }
        }

        return $return;
    }, 10, 3);
}

add_action('init', 'add_bricks_form_webhook');