<?php
/**
 * Add a custom Form Webhook element to Bricks Builder
 */
function add_bricks_form_webhook_element() {
    $element_name = 'form_webhook';

    add_filter('bricks/elements/register', function($elements) use ($element_name) {
        $elements[$element_name] = [
            'name'        => $element_name,
            'label'       => esc_html__('Form Webhook', 'bricks'),
            'icon'        => 'ti-forward',
            'category'    => 'general',
            'controls'    => [
                'webhook_url' => [
                    'tab' => 'content',
                    'label' => esc_html__('Webhook URL', 'bricks'),
                    'type' => 'text',
                    'inline' => true,
                    'placeholder' => 'https://example.com/webhook',
                ],
                'http_method' => [
                    'tab' => 'content',
                    'label' => esc_html__('HTTP Method', 'bricks'),
                    'type' => 'select',
                    'options' => [
                        'POST' => 'POST',
                        'GET' => 'GET',
                        'PUT' => 'PUT',
                        'PATCH' => 'PATCH',
                    ],
                    'default' => 'POST',
                ],
                'custom_headers' => [
                    'tab' => 'content',
                    'label' => esc_html__('Custom Headers', 'bricks'),
                    'type' => 'repeater',
                    'fields' => [
                        'key' => [
                            'label' => esc_html__('Header Key', 'bricks'),
                            'type' => 'text',
                        ],
                        'value' => [
                            'label' => esc_html__('Header Value', 'bricks'),
                            'type' => 'text',
                        ],
                    ],
                ],
            ],
        ];

        return $elements;
    });

    add_filter('bricks/form/submit', function($return, $form, $form_settings) use ($element_name) {
        $webhook_elements = \Bricks\Helpers::get_elements_by_type($element_name);

        foreach ($webhook_elements as $element) {
            $webhook_url = isset($element['settings']['webhook_url']) ? $element['settings']['webhook_url'] : '';
            if (empty($webhook_url)) continue;

            $headers = ['Content-Type' => 'application/json'];
            if (!empty($element['settings']['custom_headers']) && is_array($element['settings']['custom_headers'])) {
                foreach ($element['settings']['custom_headers'] as $header) {
                    if (!empty($header['key']) && !empty($header['value'])) {
                        $headers[$header['key']] = $header['value'];
                    }
                }
            }

            $args = [
                'body' => json_encode($form['form_fields']),
                'headers' => $headers,
                'method' => $element['settings']['http_method'] ?? 'POST',
            ];

            $response = wp_remote_request($webhook_url, $args);

            if (is_wp_error($response)) {
                error_log('Bricks Form Webhook error: ' . $response->get_error_message());
            }
        }

        return $return;
    }, 10, 3);
}

add_action('init', 'add_bricks_form_webhook_element');