<?php
/**
 * Add webhook functionality to Custom action in Bricks Builder forms
 */
function add_bricks_form_webhook_to_custom_action() {
    add_filter('bricks/form/action/custom', function($output, $settings) {
        $output['webhook_url'] = [
            'name' => 'webhook_url',
            'label' => esc_html__('Webhook URL', 'bricks'),
            'type' => 'text',
        ];
        $output['http_method'] = [
            'name' => 'http_method',
            'label' => esc_html__('HTTP Method', 'bricks'),
            'type' => 'select',
            'options' => [
                'POST' => 'POST',
                'GET' => 'GET',
                'PUT' => 'PUT',
                'PATCH' => 'PATCH',
            ],
            'default' => 'POST',
        ];
        $output['custom_headers'] = [
            'name' => 'custom_headers',
            'label' => esc_html__('Custom Headers', 'bricks'),
            'type' => 'repeater',
            'fields' => [
                'key' => [
                    'label' => esc_html__('Header Key', 'bricks'),
                    'type' => 'text',
                ],
                'value' => [
                    'label' => esc_html__('Header Value', 'bricks'),
                    'type' => 'text',
                ],
            ],
        ];
        return $output;
    }, 10, 2);

    add_action('bricks/form/submit', function($form, $form_settings) {
        if (!empty($form_settings['actions']['custom']['webhook_url'])) {
            $webhook_url = $form_settings['actions']['custom']['webhook_url'];
            $headers = ['Content-Type' => 'application/json'];
            if (!empty($form_settings['actions']['custom']['custom_headers'])) {
                foreach ($form_settings['actions']['custom']['custom_headers'] as $header) {
                    if (!empty($header['key']) && !empty($header['value'])) {
                        $headers[$header['key']] = $header['value'];
                    }
                }
            }

            $args = [
                'body' => json_encode($form['form_fields']),
                'headers' => $headers,
                'method' => $form_settings['actions']['custom']['http_method'] ?? 'POST',
            ];

            $response = wp_remote_request($webhook_url, $args);

            if (is_wp_error($response)) {
                error_log('Bricks Form Webhook error: ' . $response->get_error_message());
            }
        }
    }, 10, 2);
}

add_action('init', 'add_bricks_form_webhook_to_custom_action');