<?php
/**
 * Sendy Integration
 */
add_action('bricks/form/custom_action', 'sendy_subscription_form', 10, 1);
function sendy_subscription_form($form) {
    // Get form data
    $form_settings = $form->get_settings();
    $form_fields = $form_settings['fields'];
    $form_completed_fields = $form->get_fields();

    // Debug (uncomment if needed)
    // error_log('Form Fields: ' . print_r($form_fields, true));
    // error_log('Completed Fields: ' . print_r($form_completed_fields, true));

    // Sendy Configuration
    $sendy_url = 'https://oviewdigital.com/sendy'; // Your Sendy URL (without trailing slash)
    $api_key = 'H9rvLoWwWVbNm4xXO0Sv'; // Your API key
    $fallback_list_id = 'TO6LURSotGU5BW7NE91fNg'; // Your fallback list ID

    // Initialize variables
    $email = '';
    $sendy_list_id = '';

    // Loop through form fields to match field types with values
    foreach ($form_fields as $field) {
        $field_id = !empty($field['id']) ? $field['id'] : '';
        $field_key = 'form-field-' . $field_id;

        // Get email from email type field
        if ($field['type'] === 'email' && isset($form_completed_fields[$field_key])) {
            $email = $form_completed_fields[$field_key];
        }

        // Get Sendy List ID from hidden field with name 'sendy-list-id'
        if ($field['type'] === 'hidden' && 
            isset($field['name']) && 
            $field['name'] === 'sendy-list-id' && 
            isset($form_completed_fields['sendy-list-id'])) {
            $sendy_list_id = $form_completed_fields['sendy-list-id'];
        }
    }

    // Use fallback list ID if none found
    if (empty($sendy_list_id)) {
        $sendy_list_id = $fallback_list_id;
    }

    // Validate email
    if (empty($email) || !filter_var($email, FILTER_VALIDATE_EMAIL)) {
        return [
            'success' => false,
            'error' => 'Invalid email address'
        ];
    }

    // Prepare data for Sendy
    $postdata = array(
        'email' => $email,
        'list' => $sendy_list_id,
        'api_key' => $api_key,
        'boolean' => 'true'
    );

    // Make API request to Sendy
    $response = wp_remote_post($sendy_url . '/subscribe', array(
        'body' => $postdata,
        'timeout' => 15,
        'headers' => array(
            'Content-Type' => 'application/x-www-form-urlencoded'
        )
    ));

    // Handle response
    if (is_wp_error($response)) {
        $error_message = $response->get_error_message();
        error_log('Sendy API Error: ' . $error_message);
        return [
            'success' => false,
            'error' => 'Connection error: ' . $error_message
        ];
    }

    $body = wp_remote_retrieve_body($response);
    $http_code = wp_remote_retrieve_response_code($response);

    // Check HTTP response code
    if ($http_code !== 200) {
        error_log('Sendy API HTTP Error: ' . $http_code);
        return [
            'success' => false,
            'error' => 'Server error: ' . $http_code
        ];
    }

    // Handle Sendy response
    switch ($body) {
        case '1':
            return [
                'success' => true,
                'message' => 'Successfully subscribed!'
            ];
        case 'Already subscribed.':
            return [
                'success' => false,
                'error' => 'This email is already subscribed'
            ];
        case 'Invalid email address.':
            return [
                'success' => false,
                'error' => 'Invalid email address'
            ];
        case 'Invalid list ID.':
            return [
                'success' => false,
                'error' => 'Invalid list ID'
            ];
        case 'Invalid API key.':
            error_log('Sendy API Error: Invalid API key');
            return [
                'success' => false,
                'error' => 'Authentication failed'
            ];
        default:
            error_log('Sendy API Unknown Response: ' . $body);
            return [
                'success' => false,
                'error' => 'An unexpected error occurred'
            ];
    }
}