<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

class SIMPLIFIED_POST_WORDS_COUNT {
    public function __construct() {
        if ( is_admin() ) {
            add_filter( 'manage_posts_columns', array( $this, 'add_word_count_column' ) );
            add_action( 'manage_posts_custom_column', array( $this, 'display_word_count' ), 10, 2 );
            add_filter( 'manage_edit-post_sortable_columns', array( $this, 'make_word_count_column_sortable' ) );
            add_action( 'pre_get_posts', array( $this, 'word_count_orderby' ) );
            add_action( 'save_post', array( $this, 'update_word_count_on_save' ) );
            add_action( 'admin_init', array( $this, 'maybe_update_all_word_counts' ) );
        }
    }

    // Add custom column to post list
    public function add_word_count_column( $columns ) {
        $columns['word_count'] = __( 'Words', 'simplified-post-words-count' );
        return $columns;
    }

    // Populate custom column with word count
    public function display_word_count( $column, $post_id ) {
        if ( $column === 'word_count' ) {
            $word_count = get_post_meta( $post_id, '_word_count', true );
            if ( ! $word_count ) {
                $word_count = $this->calculate_word_count( $post_id );
                update_post_meta( $post_id, '_word_count', $word_count );
            }
            echo $word_count;
        }
    }

    // Make the column sortable
    public function make_word_count_column_sortable( $columns ) {
        $columns['word_count'] = 'word_count';
        return $columns;
    }

    // Handle sorting
    public function word_count_orderby( $query ) {
        if ( ! is_admin() ) {
            return;
        }

        $orderby = $query->get( 'orderby' );

        if ( 'word_count' === $orderby ) {
            $query->set( 'meta_key', '_word_count' );
            $query->set( 'orderby', 'meta_value_num' );
        }
    }

    // Calculate word count
    private function calculate_word_count( $post_id ) {
        $content = get_post_field( 'post_content', $post_id );
        return str_word_count( strip_tags( $content ) );
    }

    // Update word count when post is saved
    public function update_word_count_on_save( $post_id ) {
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return;
        }

        $word_count = $this->calculate_word_count( $post_id );
        update_post_meta( $post_id, '_word_count', $word_count );
    }

    // Update all word counts if needed
    public function maybe_update_all_word_counts() {
        if ( get_option( 'word_count_updated' ) !== '1' ) {
            $posts = get_posts( array(
                'post_type' => 'post',
                'posts_per_page' => -1,
            ));

            foreach ( $posts as $post ) {
                $word_count = $this->calculate_word_count( $post->ID );
                update_post_meta( $post->ID, '_word_count', $word_count );
            }

            update_option( 'word_count_updated', '1' );
        }
    }
}

new SIMPLIFIED_POST_WORDS_COUNT();